//-----------------------------------------------------------------------------
/*
Programmed by Olivier Sangala
http://olivier.sangala.free.fr
olivier_sangala@hotmail.com
*/
//-----------------------------------------------------------------------------

// C Source File
// Created 30/08/02; 20:51:59

#define USE_TI89              // Compile for TI-89
#define USE_TI92PLUS          // Compile for TI-92 Plus
#define USE_V200              // Compile for V200

// #define OPTIMIZE_ROM_CALLS // Use ROM Call Optimization

#define MIN_AMS 100           // Compile for AMS 1.00 or higher

#define SAVE_SCREEN           // Save/Restore LCD Contents

#include <tigcclib.h>         // Include All Header Files

#include <extgraph.h>

//---------------------------------------------------------------------------

#define BOARD_WIDTH (16)
#define BOARD_HEIGHT (20)

#define LEFT_STATS (85)

//-----------------------------------------------------------------------------

enum {T_UP,T_LEFT,T_DOWN,T_RIGHT,T_ROTATE_CW,T_ROTATE_ACW};
enum {COLL_OK,COLL_WALL,COLL_BLOCK};

//-----------------------------------------------------------------------------

typedef struct
{
	int MainBlockData[4][2];
	int NextBlockData[4][2];
	
	int MainBlockID;
	int NextBlockID;
	
	int MainBlockPosX;
	int MainBlockPosY;
	
	int Lines;
	int Level;
	
	int GameLost;
	
	int TimeEnable;
	
	
	int Tiles[BOARD_WIDTH][BOARD_HEIGHT];
}
TGame;

//-----------------------------------------------------------------------------

int BlockMove(int MoveWay);
void BlockDrop(void);
int BlockLoadNext(void);

void CheckForLines(void);
void NewGame(void);

void DrawTileXOR(int x, int y);
void DrawBlockXOR(void);
void DrawScreen(void);

//-----------------------------------------------------------------------------


TGame *Game;

//-----------------------------------------------------------------------------

unsigned char TileData[5] = {0x70,0x88,0x88,0x88,0x70,}; // F8 <-> 88

//-----------------------------------------------------------------------------


int BlockData[7][4][2]={
{{-1,0},{0,0},{ 1,0},{-1,1}},
{{-1,0},{0,0},{ 1,0},{ 1,1}},
{{-1,0},{0,0},{ 0,1},{ 1,1}},
{{ 0,0},{1,0},{-1,1},{ 0,1}},
{{ 0,0},{0,1},{ 1,0},{ 1,1}},
{{-1,0},{0,0},{ 1,0},{ 0,1}},
{{-1,0},{0,0},{ 1,0},{ 2,0}},
};

/*

BLOCKS

#0   #1
-------

*@* *@*
*     *

#2   #3
-------

*@   @*
 ** ** 

#4  #5  #6
------------

@*  *@* *@**
**   *      

*/

//-----------------------------------------------------------------------------


int BlockMove(int MoveWay)
{
	
	
	
	
	// PUSH DATA
	
	int MainBlockPosX=Game->MainBlockPosX;
	int MainBlockPosY=Game->MainBlockPosY;
	
	int MainBlockData[4][2];
	memcpy(MainBlockData,Game->MainBlockData,16);
	
	// HIDE BLOCK
	
	DrawBlockXOR();
	
	// MOVE BLOCK
	
	if (MoveWay==T_ROTATE_CW || MoveWay==T_ROTATE_ACW)
	{
		
		int a;
		for (a=0;a<4;a++)
		{
			int x=MainBlockData[a][0];
			int y=MainBlockData[a][1];
			
			if (MoveWay==T_ROTATE_CW)
			{
				MainBlockData[a][0]=-y;
				MainBlockData[a][1]=x;
			}
			else
			{
				MainBlockData[a][0]=y;
				MainBlockData[a][1]=-x;
			}
			
			
		}
			
	}
	else
	{
		switch(MoveWay)
		{
			case T_LEFT  : MainBlockPosX--; break;
			case T_RIGHT : MainBlockPosX++; break;
			case T_UP    : MainBlockPosY--; break;
			case T_DOWN  : MainBlockPosY++; break;
		}
	}
	
	// CHECK FOR COLLISIONS
	
	int r=COLL_OK;
	
	int a;
	for (a=0;a<4;a++)
	{
		int x=MainBlockPosX+MainBlockData[a][0];
		int y=MainBlockPosY+MainBlockData[a][1];
				
		if (x<0 || x>=BOARD_WIDTH || y<0 || y>=BOARD_HEIGHT)
		{
			r=COLL_WALL;
			break;
		}
	}
	
	if (r!=COLL_WALL)
	{
		int a;
		for (a=0;a<4;a++)
		{
			int x=MainBlockPosX+MainBlockData[a][0];
			int y=MainBlockPosY+MainBlockData[a][1];
			
			if (Game->Tiles[x][y])
			{
				r=COLL_BLOCK;
				break;
			}
		}
	}
	
	// CONFIRM MOVE
	
	if (r==COLL_OK)
	{
		// POP DATA
		
		Game->MainBlockPosX=MainBlockPosX;
		Game->MainBlockPosY=MainBlockPosY;
		
		memcpy(Game->MainBlockData,MainBlockData,16);
	}
	
	// SHOW BLOCK
	
	DrawBlockXOR();
	
	// BLOCK FIXED
	
	if (MoveWay==T_DOWN && r!=COLL_OK)
	{
		
		
		//DrawScreen();
		
		CheckForLines();
		
		int r=BlockLoadNext();
		
		DrawScreen();
		
		if (r==COLL_BLOCK)
		Game->GameLost=1;
		
	}
	
	
	return r;
	
}
//-----------------------------------------------------------------------------


void BlockDrop(void)
{
	while (BlockMove(T_DOWN)==COLL_OK);
}

//-----------------------------------------------------------------------------

int BlockLoadNext(void)
{
	
	// RANDOM NEW BLOCK
	
	Game->MainBlockID = Game->NextBlockID;
	Game->NextBlockID = random(7);
	
	// LOAD BLOCK DATA
	
	memcpy(Game->MainBlockData,&BlockData[Game->MainBlockID][0][0],16);
	memcpy(Game->NextBlockData,&BlockData[Game->NextBlockID][0][0],16);
	
	// MAIN BLOCK POSITION
	
	Game->MainBlockPosX=7;
	Game->MainBlockPosY=0;
	
	// CHECK FOR COLLISION
	
	int r=COLL_OK;
	
	int a;
	for (a=0;a<4;a++)
	{
		int x=Game->MainBlockPosX+Game->MainBlockData[a][0];
		int y=Game->MainBlockPosY+Game->MainBlockData[a][1];
		
		if (Game->Tiles[x][y])
		{
			r=COLL_BLOCK;
			break;
		}
	}
	
	DrawBlockXOR();
	
	return r;
	
}
//---------------------------------------------------------------------------


void CheckForLines(void)
{
	int x,y;
	
	for (y=0;y<BOARD_HEIGHT;y++)
	{
		
		// CHECK FOR LINE
		
		int c = 0;
		for (x=0;x<BOARD_WIDTH;x++)
		if (Game->Tiles[x][y])
		c++;
		
		// IF WE HAVE A LINE
		
		if (c==BOARD_WIDTH)
		{
			
			Game->Lines++;
			Game->Level=Game->Lines/10;

			if (Game->Level>9)
			Game->Level=9;
			
			for (x=0;x<BOARD_WIDTH;x++)
			{
				int num = Game->Tiles[x][y];
				Game->Tiles[x][y] = 0;
				if (num^0)
				DrawTileXOR(x,y);
			}
			
			int yy;
			for (yy=y;yy>0;yy--)
			{
				
				for (x=0;x<BOARD_WIDTH;x++)
				{
					int num = Game->Tiles[x][yy];
					Game->Tiles[x][yy] = Game->Tiles[x][yy-1];
					if (num^Game->Tiles[x][yy-1])
					DrawTileXOR(x,yy);
				}
				
			}
			
			for (x=0;x<BOARD_WIDTH;x++)
			{
				int num = Game->Tiles[x][0];
				Game->Tiles[x][0] = 0;
				if (num^0)
				DrawTileXOR(x,0);
			}
			
			//DrawScreen();
			
		}
		
	}
	
}
//---------------------------------------------------------------------------

void NewGame(void)
{
	
	// CLEAR BOARD
	memset(Game->Tiles,0,2*BOARD_HEIGHT*BOARD_WIDTH);
	
	Game->Level = 0;
	Game->Lines = 0;
	
	Game->GameLost=0;
	
	Game->TimeEnable=1;
	
	Game->NextBlockID = random(7);
	BlockLoadNext();
	
}
//-----------------------------------------------------------------------------


void DrawTileXOR(int x, int y)
{
	
	Sprite8_XOR((x<<2)+x,(y<<2)+y,5,TileData,LCD_MEM);
	
}
//-----------------------------------------------------------------------------

void DrawBlockXOR(void)
{
	
	int a;
	for (a=0;a<4;a++)
	{
		int x=Game->MainBlockPosX+Game->MainBlockData[a][0];
		int y=Game->MainBlockPosY+Game->MainBlockData[a][1];
		
		Game->Tiles[x][y] ^= 1;
		DrawTileXOR(x,y);
		
	}
	
}
//---------------------------------------------------------------------------


void DrawScreen(void)
{
	
	
	ClrScr();
	
	int xx=(5*BOARD_WIDTH);
	DrawLine(xx,0,xx,99,A_NORMAL);
	
	
	// BOARD
	
	int x,y;
	for (x=0;x<BOARD_WIDTH;x++)
	for (y=0;y<BOARD_HEIGHT;y++)
	if (Game->Tiles[x][y])
	DrawTileXOR(x,y);
	
	// NEXT BLOCK
	
	int a;
	for (a=0;a<4;a++)
	{
		int x=Game->NextBlockData[a][0];
		int y=Game->NextBlockData[a][1];
		
		DrawTileXOR(16+8+x,20-3+y);
	}
	
	// STATS
	
	char buffer[50];
	
	sprintf(buffer,"BLOCK OUT");
	DrawStr(LEFT_STATS,0,buffer,A_NORMAL);
	
	sprintf(buffer,"Lines : %d",Game->Lines);
	DrawStr(LEFT_STATS,20,buffer,A_NORMAL);
	
	sprintf(buffer,"Level : %d",Game->Level);
	DrawStr(LEFT_STATS,30,buffer,A_NORMAL);
	
	sprintf(buffer,"NEXT :");
	DrawStr(LEFT_STATS,70,buffer,A_NORMAL);
	
	
	
}
//-----------------------------------------------------------------------------

// Main Function
void _main(void)
{
	
	FontSetSys(0);
	clrscr();
	printf("BlockOut 1.3 \n");
	printf("Programmed by Olivier Sangala \n");
	printf("http://olivier.sangala.free.fr \n");
	printf("olivier_sangala@hotmail.com \n");
	FontSetSys(1);
	ngetchx();
	
	
	randomize();
	
	FontSetSys(0);
	
	TGame GameA;
	Game=&GameA;
	
	OSFreeTimer (USER_TIMER);
	OSRegisterTimer (USER_TIMER, 40-(4*Game->Level));
	
	
	NewGame();
	DrawScreen();
	
	
	
	for (;;)
	{
		
		
		
		// LEVEL #0 : 2s   -> 40 ticks
		// LEVEL #9 : 1/5s -> 4  ticks
		// FORMULA :  ticks = 40 - ( 4 * level )
		
		while (!kbhit())
		{
			if (Game->TimeEnable)
			{
				if (OSTimerExpired (USER_TIMER))
				{
					BlockMove(T_DOWN);
					
					OSFreeTimer (USER_TIMER);
					OSRegisterTimer (USER_TIMER, 40-(4*Game->Level));
				}
			}
			
			
			
			if (Game->GameLost)
			{
				DrawStr(LEFT_STATS,50,"YOU LOST!",A_NORMAL);
				GKeyFlush();
				ngetchx();
				
				NewGame();
				DrawScreen();
			}
		}
		
		int key=ngetchx();
		
		
		if (key==KEY_RIGHT)
		BlockMove(T_RIGHT);
		else if (key==KEY_LEFT)
		BlockMove(T_LEFT);
		else if (key==KEY_DOWN)
		BlockMove(T_DOWN);
		/*
		else if (key==KEY_UP)
		BlockMove(T_UP);
		*/
		
		
		else if (key==KEY_F1)
		BlockMove(T_ROTATE_CW);
		else if (key==KEY_F2)
		BlockMove(T_ROTATE_ACW);
		
		else if (key==KEY_F3)
		BlockDrop();
		
		
		else if (key==KEY_APPS)
		off();
		
		else if (key==KEY_ESC)
		break;
		
		
	}
	
	FontSetSys(1);
	
	
}
